@@ -1,22 +1,9 @@
 package Archive::Zip;
 
-# Copyright 2000 - 2002 Ned Konz. All rights reserved. This program is free
-# software; you can redistribute it and/or modify it under the same terms as
-# Perl itself.
-
-# ----------------------------------------------------------------------
-# class Archive::Zip
-# Note that the package Archive::Zip exists only for exporting and
-# sharing constants. Everything else is in another package
-# in this file.
-# Creation of a new Archive::Zip object actually creates a new object
-# of class Archive::Zip::Archive.
-# ----------------------------------------------------------------------
-
+use strict;
 BEGIN {
 	require 5.003_96;
 }
-use strict;
 use UNIVERSAL      ();
 use Carp           ();
 use IO::File       ();
@@ -24,24 +11,25 @@ use IO::Seekable   ();
 use Compress::Zlib ();
 use File::Spec     ();
 use File::Temp     ();
+use FileHandle     ();
 
 use vars qw( $VERSION @ISA );
 BEGIN {
-	$VERSION = '1.17_05';
-	$VERSION = eval $VERSION;
+	$VERSION = '1.27_01';
 
 	require Exporter;
 	@ISA = qw( Exporter );
 }
 
 use vars qw( $ChunkSize $ErrorHandler );
+BEGIN {
+	# This is the size we'll try to read, write, and (de)compress.
+	# You could set it to something different if you had lots of memory
+	# and needed more speed.
+	$ChunkSize ||= 32768;
 
-# This is the size we'll try to read, write, and (de)compress.
-# You could set it to something different if you had lots of memory
-# and needed more speed.
-$ChunkSize = 32768;
-
-$ErrorHandler = \&Carp::carp;
+	$ErrorHandler = \&Carp::carp;
+}
 
 # BEGIN block is necessary here so that other modules can use the constants.
 use vars qw( @EXPORT_OK %EXPORT_TAGS );
@@ -273,14 +261,32 @@ require Archive::Zip::StringMember;
 use constant ZIPARCHIVECLASS => 'Archive::Zip::Archive';
 use constant ZIPMEMBERCLASS  => 'Archive::Zip::Member';
 
-sub new    
-{
+# Convenience functions
+
+sub _ISA ($$) {
+	# Can't rely on Scalar::Util, so use the next best way
+	local $@;
+	!! eval { ref $_[0] and $_[0]->isa($_[1]) };
+}
+
+sub _CAN ($$) {
+	local $@;
+	!! eval { ref $_[0] and $_[0]->can($_[1]) };
+}
+
+
+
+
+
+#####################################################################
+# Methods
+
+sub new {
 	my $class = shift;
 	return $class->ZIPARCHIVECLASS->new(@_);
 }
 
-sub computeCRC32    
-{
+sub computeCRC32 {
 	my $data = shift;
 	$data = shift if ref($data);    # allow calling as an obj method
 	my $crc = shift;
@@ -289,8 +295,7 @@ sub computeCRC32
 
 # Report or change chunk size used for reading and writing.
 # Also sets Zlib's default buffer size (eventually).
-sub setChunkSize    
-{
+sub setChunkSize {
 	my $chunkSize = shift;
 	$chunkSize = shift if ref($chunkSize);    # object method on zip?
 	my $oldChunkSize = $Archive::Zip::ChunkSize;
@@ -298,13 +303,11 @@ sub setChunkSize
 	return $oldChunkSize;
 }
 
-sub chunkSize    
-{
+sub chunkSize {
 	return $Archive::Zip::ChunkSize;
 }
 
-sub setErrorHandler (&)    
-{
+sub setErrorHandler (&) {
 	my $errorHandler = shift;
 	$errorHandler = \&Carp::carp unless defined($errorHandler);
 	my $oldErrorHandler = $Archive::Zip::ErrorHandler;
@@ -312,12 +315,14 @@ sub setErrorHandler (&)
 	return $oldErrorHandler;
 }
 
-# ----------------------------------------------------------------------
-# Private utility functions (not methods).
-# ----------------------------------------------------------------------
 
-sub _printError    
-{
+
+
+
+######################################################################
+# Private utility functions (not methods).
+
+sub _printError {
 	my $string = join ( ' ', @_, "\n" );
 	my $oldCarpLevel = $Carp::CarpLevel;
 	$Carp::CarpLevel += 2;
@@ -326,24 +331,21 @@ sub _printError
 }
 
 # This is called on format errors.
-sub _formatError    
-{
+sub _formatError {
 	shift if ref( $_[0] );
 	_printError( 'format error:', @_ );
 	return AZ_FORMAT_ERROR;
 }
 
 # This is called on IO errors.
-sub _ioError    
-{
+sub _ioError {
 	shift if ref( $_[0] );
 	_printError( 'IO error:', @_, ':', $! );
 	return AZ_IO_ERROR;
 }
 
 # This is called on generic errors.
-sub _error    
-{
+sub _error {
 	shift if ref( $_[0] );
 	_printError( 'error:', @_ );
 	return AZ_ERROR;
@@ -351,78 +353,78 @@ sub _error
 
 # Called when a subclass should have implemented
 # something but didn't
-sub _subclassResponsibility    
-{
+sub _subclassResponsibility {
 	Carp::croak("subclass Responsibility\n");
 }
 
 # Try to set the given file handle or object into binary mode.
-sub _binmode    
-{
+sub _binmode {
 	my $fh = shift;
-	return UNIVERSAL::can( $fh, 'binmode' ) ? $fh->binmode() : binmode($fh);
+	return _CAN( $fh, 'binmode' ) ? $fh->binmode() : binmode($fh);
 }
 
 # Attempt to guess whether file handle is seekable.
-# Because of problems with Windoze, this only returns true when
+# Because of problems with Windows, this only returns true when
 # the file handle is a real file.  
-sub _isSeekable    
-{
+sub _isSeekable {
 	my $fh = shift;
-	if ( UNIVERSAL::isa($fh, 'IO::Scalar') ) {
+	return 0 unless ref $fh;
+	if ( _ISA($fh, 'IO::Scalar') ) {
+		# IO::Scalar objects are brokenly-seekable
 		return 0;
 	}
-	if ( UNIVERSAL::isa($fh, 'IO::String') ) {
+	if ( _ISA($fh, 'IO::String') ) {
 		return 1;
 	}
-	if ( UNIVERSAL::isa($fh, 'IO::Seekable') ) {
+	if ( _ISA($fh, 'IO::Seekable') ) {
 		# Unfortunately, some things like FileHandle objects
 		# return true for Seekable, but AREN'T!!!!!
-		if ( UNIVERSAL::isa($fh, 'FileHandle') ) {
+		if ( _ISA($fh, 'FileHandle') ) {
 			return 0;
+		} else {
+			return 1;
 		}
-		return 1;
 	}
-	if ( UNIVERSAL::can($fh, 'stat') ) {
+	if ( _CAN($fh, 'stat') ) {
 		return -f $fh;
 	}
 	return (
-		UNIVERSAL::can($fh, 'seek') and UNIVERSAL::can($fh, 'tell')
+		_CAN($fh, 'seek') and _CAN($fh, 'tell')
 		) ? 1 : 0;
 }
 
+# Print to the filehandle, while making sure the pesky Perl special global 
+# variables don't interfere.
+sub _print
+{
+    my ($self, $fh, @data) = @_;
+
+    local $\;
+
+    return $fh->print(@data);
+}
+
 # Return an opened IO::Handle
 # my ( $status, fh ) = _newFileHandle( 'fileName', 'w' );
 # Can take a filename, file handle, or ref to GLOB
 # Or, if given something that is a ref but not an IO::Handle,
 # passes back the same thing.
-sub _newFileHandle    
-{
+sub _newFileHandle {
 	my $fd     = shift;
 	my $status = 1;
 	my $handle;
 
-	if ( ref($fd) )
-	{
-		if ( UNIVERSAL::isa( $fd, 'IO::Scalar' )
-			or UNIVERSAL::isa( $fd, 'IO::String' ) )
-		{
+	if ( ref($fd) ) {
+		if ( _ISA($fd, 'IO::Scalar') or _ISA($fd, 'IO::String') ) {
 			$handle = $fd;
-		}
-		elsif ( UNIVERSAL::isa( $fd, 'IO::Handle' )
-			or UNIVERSAL::isa( $fd, 'GLOB' ) )
-		{
-			$handle = IO::File->new();
+		} elsif ( _ISA($fd, 'IO::Handle') or ref($fd) eq 'GLOB' ) {
+			$handle = IO::File->new;
 			$status = $handle->fdopen( $fd, @_ );
-		}
-		else
-		{
+		} else {
 			$handle = $fd;
 		}
-	}
-	else
-	{
-		$handle = IO::File->new();
+	} else {
+		$handle = IO::File->new;
 		$status = $handle->open( $fd, @_ );
 	}
 
@@ -434,16 +436,16 @@ sub _newFileHandle
 # In list context, returns ($status, $signature)
 # ( $status, $signature) = _readSignature( $fh, $fileName );
 
-sub _readSignature    
-{
+sub _readSignature {
 	my $fh                = shift;
 	my $fileName          = shift;
 	my $expectedSignature = shift;    # optional
 
 	my $signatureData;
 	my $bytesRead = $fh->read( $signatureData, SIGNATURE_LENGTH );
-	return _ioError("reading header signature")
-	  if $bytesRead != SIGNATURE_LENGTH;
+	if ( $bytesRead != SIGNATURE_LENGTH ) {
+		return _ioError("reading header signature");
+	}
 	my $signature = unpack( SIGNATURE_FORMAT, $signatureData );
 	my $status    = AZ_OK;
 
@@ -476,8 +478,7 @@ sub _readSignature
 # my ($fh, $name) = Archive::Zip::tempFile('mytempdir');
 #
 
-sub tempFile    
-{
+sub tempFile {
 	my $dir = shift;
 	my ( $fh, $filename ) = File::Temp::tempfile(
 		SUFFIX => '.zip',
@@ -516,8 +517,8 @@ sub _asZipDirName
 	  File::Spec->splitpath( File::Spec->canonpath($name), $forceDir );
 	$$volReturn = $volume if ( ref($volReturn) );
 	my @dirs = map { $_ =~ s{/}{_}g; $_ } File::Spec->splitdir($directories);
-	if ( @dirs > 0 ) { pop (@dirs) unless $dirs[-1] }   # remove empty component
-	push ( @dirs, $file || '' );
+    if ( @dirs > 0 ) { pop (@dirs) unless $dirs[-1] }   # remove empty component
+    push ( @dirs, defined($file) ? $file : '' );
 	#return wantarray ? @dirs : join ( '/', @dirs );
     return join ( '/', @dirs );
 }
@@ -537,7 +538,8 @@ sub _asLocalName
 	$filename = '' unless defined($filename);
 	my $localDirs = @paths?File::Spec->catdir(@paths):'';
 	my $localName = File::Spec->catpath( $volume, $localDirs, $filename );
-	$localName = File::Spec->rel2abs($localName) unless $volume;
+    use Cwd;
+    $localName = File::Spec->catfile(getcwd, $localName) unless $volume;
 	return $localName;
 }
 
@@ -603,6 +605,9 @@ directories, files, or strings.
 This module uses the L<Compress::Zlib> library to read and write the
 compressed streams inside the files.
 
+One can use L<Archive::Zip::MemberRead> to read the zip file archive members
+as if they were files.
+
 =head2 File Naming
 
 Regardless of what your local file system uses for file naming, names in a
@@ -1223,7 +1228,7 @@ set if the Perl C<-f> test returns true. This could fail on
 some operating systems, though.
 
     my $fh = IO::File->new( 'someFile.zip', 'w' );
-    unless ( $zip->writeToFileHandle( $fh ) != AZ_OK ) {
+    unless ( $zip->writeToFileHandle( $fh ) == AZ_OK ) {
         # error handling
     }
 
@@ -1949,8 +1954,6 @@ L<File::Path>
 
 L<File::Spec>
 
-L<File::Spec>
-
 L<IO::File>
 
 L<IO::Seekable>
@@ -1959,6 +1962,14 @@ L<Time::Local>
 
 =head1 BUGS AND CAVEATS
 
+=head2 When not to use Archive::Zip
+
+If you are just going to be extracting zips (and/or other archives) you
+are recommended to look at using L<Archive::Extract> instead, as it is much
+easier to use and factors out archive-specific functionality.
+
+=head2 Try to avoid IO::Scalar
+
 One of the most common ways to use Archive::Zip is to generate Zip files
 in-memory. Most people have use L<IO::Scalar> for this purpose.
 
@@ -2004,31 +2015,34 @@ For other issues contact the maintainer
 
 =head1 AUTHOR
 
-Ned Konz E<lt>nedkonz@cpan.orgE<gt>
+Adam Kennedy E<lt>adamk@cpan.orgE<gt>
 
-Currently maintained by Adam Kennedy E<lt>adamk@cpan.orgE<gt>
+Previously maintained by Steve Peters E<lt>steve@fisharerojo.orgE<gt>.
 
-Previously maintained by Steve Peters E<lt>steve@fisharerojo.orgE<gt>
+File attributes code by Maurice Aubrey E<lt>maurice@lovelyfilth.comE<gt>.
 
-File attributes code by Maurice Aubrey E<lt>maurice@lovelyfilth.comE<gt>
+Originally by Ned Konz E<lt>nedkonz@cpan.orgE<gt>.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2000-2004 Ned Konz. All rights reserved.
+Some parts copyright 2006 - 2008 Adam Kennedy.
 
-Some parts copyright (c) 2005 Steve Peters. All rights reserved.
+Some parts copyright 2005 Steve Peters.
 
-Some parts copyright (c) 2006 Adam Kennedy. All rights reserved. 
+Original work copyright 2000 - 2004 Ned Konz.
 
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
+This program is free software; you can redistribute it and/or modify
+it under the same terms as Perl itself.
 
 =head1 SEE ALSO
 
+Look at L<Archive::Zip::MemberRead> which is a wrapper that allows one to
+read Zip archive members as if they were files.
+
 L<Compress::Zlib>, L<Archive::Tar>, L<Archive::Extract>
 
 There is a Japanese translation of this
-document at L<http://www.memb.jp/~deq/perl/doc-ja/Archive-Zip.html> that
-was done by DEQ E<lt>deq@oct.zaq.ne.jpE<gt> . Thanks! 
+document at L<http://www.memb.jp/~deq/perl/doc-ja/Archive-Zip.html>
+that was done by DEQ E<lt>deq@oct.zaq.ne.jpE<gt> . Thanks! 
 
 =cut
