@@ -6,7 +6,7 @@ use strict;
 use vars qw( $VERSION @ISA );
 
 BEGIN {
-    $VERSION = '1.17_05';
+    $VERSION = '1.27_01';
     @ISA     = qw( Archive::Zip );
 }
 
@@ -81,6 +81,7 @@ sub new {
         'crc32'                    => 0,
         'compressedSize'           => 0,
         'uncompressedSize'         => 0,
+        'isSymbolicLink'           => 0,
         @_
     };
     bless( $self, $class );
@@ -115,7 +116,19 @@ sub versionNeededToExtract {
 }
 
 sub bitFlag {
-    shift->{'bitFlag'};
+    my $self = shift;
+
+    # Set General Purpose Bit Flags according to the desiredCompressionLevel setting
+    if ( $self->desiredCompressionLevel == 1 || $self->desiredCompressionLevel == 2 ) {
+        $self->{'bitFlag'} = DEFLATING_COMPRESSION_FAST;
+    } elsif ( $self->desiredCompressionLevel == 3 || $self->desiredCompressionLevel == 4
+          || $self->desiredCompressionLevel == 5 || $self->desiredCompressionLevel == 6
+          || $self->desiredCompressionLevel == 7 ) {
+        $self->{'bitFlag'} = DEFLATING_COMPRESSION_NORMAL;
+    } elsif ( $self->desiredCompressionLevel == 8 || $self->desiredCompressionLevel == 9 ) {
+        $self->{'bitFlag'} = DEFLATING_COMPRESSION_MAXIMUM;
+    }
+    $self->{'bitFlag'};
 }
 
 sub compressionMethod {
@@ -130,8 +143,9 @@ sub desiredCompressionMethod {
         $self->{'desiredCompressionMethod'} = $newDesiredCompressionMethod;
         if ( $newDesiredCompressionMethod == COMPRESSION_STORED ) {
             $self->{'desiredCompressionLevel'} = 0;
-        }
-        elsif ( $oldDesiredCompressionMethod == COMPRESSION_STORED ) {
+            $self->{'bitFlag'} &= ~GPBF_HAS_DATA_DESCRIPTOR_MASK;
+
+        } elsif ( $oldDesiredCompressionMethod == COMPRESSION_STORED ) {
             $self->{'desiredCompressionLevel'} = COMPRESSION_LEVEL_DEFAULT;
         }
     }
@@ -189,12 +203,20 @@ sub externalFileAttributes {
 }
 
 # Convert UNIX permissions into proper value for zip file
-# NOT A METHOD!
+# Usable as a function or a method
 sub _mapPermissionsFromUnix {
-    my $perms = shift;
-    return $perms << 16;
+    my $self    = shift;
+    my $mode    = shift;
+    my $attribs = $mode << 16;
 
-    # TODO: map MS-DOS perms too (RHSA?)
+    # Microsoft Windows Explorer needs this bit set for directories
+    if ( $mode & DIRECTORY_ATTRIB ) {
+        $attribs |= 16;
+    }
+
+    return $attribs;
+
+    # TODO: map more MS-DOS perms
 }
 
 # Convert ZIP permissions into Unix ones
@@ -284,18 +306,17 @@ sub _mapPermissionsToUnix {
 
 sub unixFileAttributes {
     my $self     = shift;
-    my $oldPerms = $self->_mapPermissionsToUnix();
-    if (@_) {
+    my $oldPerms = $self->_mapPermissionsToUnix;
+    if ( @_ ) {
         my $perms = shift;
-        if ( $self->isDirectory() ) {
+        if ( $self->isDirectory ) {
             $perms &= ~FILE_ATTRIB;
             $perms |= DIRECTORY_ATTRIB;
-        }
-        else {
+        } else {
             $perms &= ~DIRECTORY_ATTRIB;
             $perms |= FILE_ATTRIB;
         }
-        $self->{'externalFileAttributes'} = _mapPermissionsFromUnix($perms);
+        $self->{externalFileAttributes} = $self->_mapPermissionsFromUnix($perms);
     }
     return $oldPerms;
 }
@@ -382,14 +403,48 @@ sub isBinaryFile {
 sub extractToFileNamed {
     my $self = shift;
     my $name = shift;    # local FS name
-    return _error("encryption unsupported") if $self->isEncrypted();
-    mkpath( dirname($name) );    # croaks on error
-    my ( $status, $fh ) = _newFileHandle( $name, 'w' );
-    return _ioError("Can't open file $name for write") unless $status;
+    $self->{'isSymbolicLink'} = 0;
+
+    # Check if the file / directory is a symbolic link or not
+    if ( $self->{'externalFileAttributes'} == 2717843456 ) {
+        $self->{'isSymbolicLink'} = 1;
+        $self->{'newName'} = $name;
+        my ( $status, $fh ) = _newFileHandle( $name, 'r' );
+        my $retval = $self->extractToFileHandle($fh);
+        $fh->close();
+    } else {
+        #return _writeSymbolicLink($self, $name) if $self->isSymbolicLink();
+        return _error("encryption unsupported") if $self->isEncrypted();
+        mkpath( dirname($name) );    # croaks on error
+        my ( $status, $fh ) = _newFileHandle( $name, 'w' );
+        return _ioError("Can't open file $name for write") unless $status;
+        my $retval = $self->extractToFileHandle($fh);
+        $fh->close();
+        chmod ($self->unixFileAttributes(), $name)
+            or return _error("Can't chmod() ${name}: $!");
+        utime( $self->lastModTime(), $self->lastModTime(), $name );
+        return $retval;
+    }
+}
+
+sub _writeSymbolicLink {
+    my $self = shift;
+    my $name = shift;
+    my $chunkSize = $Archive::Zip::ChunkSize;
+    #my ( $outRef, undef ) = $self->readChunk($chunkSize);
+    my $fh;
     my $retval = $self->extractToFileHandle($fh);
-    $fh->close();
-    utime( $self->lastModTime(), $self->lastModTime(), $name );
-    return $retval;
+    my ( $outRef, undef ) = $self->readChunk(100);
+}
+
+sub isSymbolicLink {
+    my $self = shift;
+    if ( $self->{'externalFileAttributes'} == 2717843456 ) {
+        $self->{'isSymbolicLink'} = 1;
+    } else {
+        return 0;
+    }
+    1;
 }
 
 sub isDirectory {
@@ -511,7 +566,7 @@ sub _writeLocalFileHeader {
     my $fh   = shift;
 
     my $signatureData = pack( SIGNATURE_FORMAT, LOCAL_FILE_HEADER_SIGNATURE );
-    $fh->print($signatureData)
+    $self->_print($fh, $signatureData)
       or return _ioError("writing local header signature");
 
     my $header = pack(
@@ -527,13 +582,15 @@ sub _writeLocalFileHeader {
         length( $self->localExtraField() )
     );
 
-    $fh->print($header) or return _ioError("writing local header");
-    if ( $self->fileName() ) {
-        $fh->print( $self->fileName() )
+    $self->_print($fh, $header) or return _ioError("writing local header");
+
+    # Check for a valid filename or a filename equal to a literal `0'
+    if ( $self->fileName() || $self->fileName eq '0' ) {
+        $self->_print($fh, $self->fileName() )
           or return _ioError("writing local header filename");
     }
     if ( $self->localExtraField() ) {
-        $fh->print( $self->localExtraField() )
+        $self->_print($fh, $self->localExtraField() )
           or return _ioError("writing local extra field");
     }
 
@@ -546,7 +603,7 @@ sub _writeCentralDirectoryFileHeader {
 
     my $sigData =
       pack( SIGNATURE_FORMAT, CENTRAL_DIRECTORY_FILE_HEADER_SIGNATURE );
-    $fh->print($sigData)
+    $self->_print($fh, $sigData)
       or return _ioError("writing central directory header signature");
 
     my $fileNameLength    = length( $self->fileName() );
@@ -573,18 +630,18 @@ sub _writeCentralDirectoryFileHeader {
         $self->writeLocalHeaderRelativeOffset()
     );
 
-    $fh->print($header)
+    $self->_print($fh, $header)
       or return _ioError("writing central directory header");
     if ($fileNameLength) {
-        $fh->print( $self->fileName() )
+        $self->_print($fh,  $self->fileName() )
           or return _ioError("writing central directory header signature");
     }
     if ($extraFieldLength) {
-        $fh->print( $self->cdExtraField() )
+        $self->_print($fh,  $self->cdExtraField() )
           or return _ioError("writing central directory extra field");
     }
     if ($fileCommentLength) {
-        $fh->print( $self->fileComment() )
+        $self->_print($fh,  $self->fileComment() )
           or return _ioError("writing central directory file comment");
     }
 
@@ -607,7 +664,7 @@ sub _writeDataDescriptor {
         $self->uncompressedSize()
     );
 
-    $fh->print($header)
+    $self->_print($fh, $header)
       or return _ioError("writing data descriptor");
     return AZ_OK;
 }
@@ -637,7 +694,7 @@ sub _refreshLocalFileHeader {
         length( $self->localExtraField() )
     );
 
-    $fh->print($header)
+    $self->_print($fh, $header)
       or return _ioError("re-writing local header");
     $fh->seek( $here, IO::Seekable::SEEK_SET )
       or return _ioError("seeking after rewrite of local header");
@@ -877,7 +934,7 @@ sub _writeToFileHandle {
     my $offset       = shift;
 
     return _error("no member name given for $self")
-      unless $self->fileName();
+      if $self->fileName() eq '';
 
     $self->{'writeLocalHeaderRelativeOffset'} = $offset;
     $self->{'wasWritten'}                     = 0;
@@ -923,22 +980,30 @@ sub _writeData {
     my $self    = shift;
     my $writeFh = shift;
 
-    return AZ_OK if ( $self->uncompressedSize() == 0 );
-    my $status;
-    my $chunkSize = $Archive::Zip::ChunkSize;
-    while ( $self->_readDataRemaining() > 0 ) {
-        my $outRef;
-        ( $outRef, $status ) = $self->readChunk($chunkSize);
-        return $status if ( $status != AZ_OK and $status != AZ_STREAM_END );
+    # If symbolic link, just create one if the operating system is Linux, Unix, BSD or VMS
+    # TODO: Add checks for other operating systems
+    if ( $self->{'isSymbolicLink'} == 1 && $^O eq 'linux' ) {
+        my $chunkSize = $Archive::Zip::ChunkSize;
+        my ( $outRef, $status ) = $self->readChunk($chunkSize);
+        symlink $$outRef, $self->{'newName'};
+    } else {
+        return AZ_OK if ( $self->uncompressedSize() == 0 );
+        my $status;
+        my $chunkSize = $Archive::Zip::ChunkSize;
+        while ( $self->_readDataRemaining() > 0 ) {
+            my $outRef;
+            ( $outRef, $status ) = $self->readChunk($chunkSize);
+            return $status if ( $status != AZ_OK and $status != AZ_STREAM_END );
 
-        if ( length($$outRef) > 0 ) {
-            $writeFh->print($$outRef)
-              or return _ioError("write error during copy");
+            if ( length($$outRef) > 0 ) {
+                $self->_print($writeFh, $$outRef)
+                  or return _ioError("write error during copy");
+            }
+
+            last if $status == AZ_STREAM_END;
         }
-
-        last if $status == AZ_STREAM_END;
+        $self->{'compressedSize'} = $self->_writeOffset();
     }
-    $self->{'compressedSize'} = $self->_writeOffset();
     return AZ_OK;
 }
 
