@@ -1,6 +1,16 @@
+package Archive::Zip::Tree;
+
+use strict;
+use vars qw{$VERSION};
+BEGIN {
+	$VERSION = '1.27_01';
+}
+
 use Archive::Zip;
 
-warn("Archive::Zip::Tree is deprecated; its methods have been moved into Archive::Zip.") if $^W;
+warn(
+"Archive::Zip::Tree is deprecated; its methods have been moved into Archive::Zip."
+) if $^W;
 
 1;
 
@@ -36,3 +46,4 @@ as Perl itself.
 L<Archive::Zip>
 
 =cut
+
